@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module RBS
   module Prototype
     class Runtime
@@ -7,6 +9,7 @@ module RBS
       attr_reader :env
       attr_reader :merge
       attr_reader :owners_included
+      attr_accessor :outline
 
       def initialize(patterns:, env:, merge:, owners_included: [])
         @patterns = patterns
@@ -17,10 +20,12 @@ module RBS
         @owners_included = owners_included.map do |name|
           Object.const_get(name)
         end
+        @outline = false
       end
 
       def target?(const)
         name = const_name(const)
+        return false unless name
 
         patterns.any? do |pattern|
           if pattern.end_with?("*")
@@ -77,7 +82,7 @@ module RBS
           supers.merge(mix.included_modules)
         end
 
-        if mod.is_a?(Class)
+        if mod.is_a?(Class) && mod.superclass
           mod.superclass.included_modules.each do |mix|
             supers << mix
             supers.merge(mix.included_modules)
@@ -135,7 +140,8 @@ module RBS
           when :block
             block = Types::Block.new(
               type: Types::Function.empty(untyped).update(rest_positionals: Types::Function::Param.new(name: nil, type: untyped)),
-              required: true
+              required: true,
+              self_type: nil
             )
           end
         end
@@ -183,18 +189,22 @@ module RBS
             if method
               members << AST::Members::MethodDefinition.new(
                 name: method_name,
-                types: method.method_types.map {|type|
-                  type.update.tap do |ty|
-                    def ty.to_s
-                      location.source
+                overloads: method.method_types.map {|type|
+                  AST::Members::MethodDefinition::Overload.new(
+                    annotations: [],
+                    method_type: type.update.tap do |ty|
+                      def ty.to_s
+                        location.source
+                      end
                     end
-                  end
+                  )
                 },
                 kind: kind,
                 location: nil,
                 comment: method.comments[0],
                 annotations: method.annotations,
-                overload: false
+                overloading: false,
+                visibility: nil
               )
               return
             end
@@ -227,12 +237,15 @@ module RBS
 
               members << AST::Members::MethodDefinition.new(
                 name: method.name,
-                types: [method_type(method)],
+                overloads: [
+                  AST::Members::MethodDefinition::Overload.new(annotations: [], method_type: method_type(method))
+                ],
                 kind: :singleton,
                 location: nil,
                 comment: nil,
                 annotations: [],
-                overload: false
+                overloading: false,
+                visibility: nil
               )
             end
           else
@@ -260,12 +273,15 @@ module RBS
 
                 members << AST::Members::MethodDefinition.new(
                   name: method.name,
-                  types: [method_type(method)],
+                  overloads: [
+                    AST::Members::MethodDefinition::Overload.new(annotations: [], method_type: method_type(method))
+                  ],
                   kind: :instance,
                   location: nil,
                   comment: nil,
                   annotations: [],
-                  overload: false
+                  overloading: false,
+                  visibility: nil
                 )
               end
             else
@@ -294,12 +310,15 @@ module RBS
 
                 members << AST::Members::MethodDefinition.new(
                   name: method.name,
-                  types: [method_type(method)],
+                  overloads: [
+                    AST::Members::MethodDefinition::Overload.new(annotations: [], method_type: method_type(method))
+                  ],
                   kind: :instance,
                   location: nil,
                   comment: nil,
                   annotations: [],
-                  overload: false
+                  overloading: false,
+                  visibility: nil
                 )
               end
             else
@@ -318,7 +337,12 @@ module RBS
 
       def generate_constants(mod, decls)
         mod.constants(false).sort.each do |name|
-          value = mod.const_get(name)
+          begin
+            value = mod.const_get(name)
+          rescue StandardError, LoadError => e
+            RBS.logger.warn("Skipping constant #{name} of #{mod} since #{e}")
+            next
+          end
 
           next if value.is_a?(Class) || value.is_a?(Module)
           unless value.class.name
@@ -350,13 +374,13 @@ module RBS
       end
 
       def generate_super_class(mod)
-        if mod.superclass == ::Object
+        if mod.superclass.nil? || mod.superclass == ::Object
           nil
         elsif const_name(mod.superclass).nil?
           RBS.logger.warn("Skipping anonymous superclass #{mod.superclass} of #{mod}")
           nil
         else
-          super_name = to_type_name(const_name(mod.superclass), full_name: true)
+          super_name = to_type_name(const_name(mod.superclass), full_name: true).absolute!
           super_args = type_args(super_name)
           AST::Declarations::Class::Super.new(name: super_name, args: super_args, location: nil)
         end
@@ -371,7 +395,7 @@ module RBS
         unless decl
           decl = AST::Declarations::Class.new(
             name: to_type_name(only_name(mod)),
-            type_params: [],
+            type_params: type_params(mod),
             super_class: generate_super_class(mod),
             members: [],
             annotations: [],
@@ -404,7 +428,7 @@ module RBS
           )
         end
 
-        generate_methods(mod, type_name, decl.members)
+        generate_methods(mod, type_name, decl.members) unless outline
 
         generate_constants mod, decl.members
       end
@@ -425,7 +449,7 @@ module RBS
         unless decl
           decl = AST::Declarations::Module.new(
             name: to_type_name(only_name(mod)),
-            type_params: [],
+            type_params: type_params(mod),
             self_types: [],
             members: [],
             annotations: [],
@@ -458,7 +482,7 @@ module RBS
           )
         end
 
-        generate_methods(mod, type_name, decl.members)
+        generate_methods(mod, type_name, decl.members) unless outline
 
         generate_constants mod, decl.members
       end
@@ -480,7 +504,7 @@ module RBS
             if outer_module.is_a?(Class)
               outer_decl = AST::Declarations::Class.new(
                 name: to_type_name(outer_module_name),
-                type_params: [],
+                type_params: type_params(outer_module),
                 super_class: generate_super_class(outer_module),
                 members: [],
                 annotations: [],
@@ -490,7 +514,7 @@ module RBS
             else
               outer_decl = AST::Declarations::Module.new(
                 name: to_type_name(outer_module_name),
-                type_params: [],
+                type_params: type_params(outer_module),
                 self_types: [],
                 members: [],
                 annotations: [],
@@ -517,7 +541,17 @@ module RBS
 
       def const_name(const)
         @module_name_method ||= Module.instance_method(:name)
-        @module_name_method.bind(const).call
+        name = @module_name_method.bind(const).call
+        return nil unless name
+
+        begin
+          Object.const_get(name)
+        rescue NameError
+          # Should generate const name if anonymous or internal module (e.g. NameError::message)
+          nil
+        else
+          name
+        end
       end
 
       def type_args(type_name)
@@ -528,6 +562,15 @@ module RBS
         end
       end
 
+      def type_params(mod)
+        type_name = to_type_name(const_name(mod), full_name: true)
+        if class_decl = env.class_decls[type_name.absolute!]
+          class_decl.type_params
+        else
+          []
+        end
+      end
+
       def block_from_ast_of(method)
         return nil if RUBY_VERSION < '3.1'
 
