@@ -47,12 +47,16 @@ static VALUE sGroup;
 #define HAVE_UNAME 1
 #endif
 
-#ifndef _WIN32
-char *getenv();
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-char *getlogin();
+char *getlogin(void);
 
-#define RUBY_ETC_VERSION "1.4.0"
+#define RUBY_ETC_VERSION "1.4.2"
 
 #ifdef HAVE_RB_DEPRECATE_CONSTANT
 void rb_deprecate_constant(VALUE mod, const char *name);
@@ -62,6 +66,17 @@ void rb_deprecate_constant(VALUE mod, const char *name);
 
 #include "constdefs.h"
 
+#ifndef HAVE_RB_IO_DESCRIPTOR
+static int
+io_descriptor_fallback(VALUE io)
+{
+    rb_io_t *fptr;
+    GetOpenFile(io, fptr);
+    return fptr->fd;
+}
+#define rb_io_descriptor io_descriptor_fallback
+#endif
+
 #ifdef HAVE_RUBY_ATOMIC_H
 # include "ruby/atomic.h"
 #else
@@ -937,14 +952,11 @@ io_pathconf(VALUE io, VALUE arg)
 {
     int name;
     long ret;
-    rb_io_t *fptr;
 
     name = NUM2INT(arg);
 
-    GetOpenFile(io, fptr);
-
     errno = 0;
-    ret = fpathconf(fptr->fd, name);
+    ret = fpathconf(rb_io_descriptor(io), name);
     if (ret == -1) {
         if (errno == 0) /* no limit */
             return Qnil;
