@@ -1,10 +1,12 @@
 # frozen_string_literal: true
+
 require_relative "../command"
 require_relative "../install_update_options"
 require_relative "../dependency_installer"
 require_relative "../local_remote_options"
 require_relative "../validator"
 require_relative "../version_option"
+require_relative "../update_suggestion"
 
 ##
 # Gem installer command line tool
@@ -17,6 +19,7 @@ class Gem::Commands::InstallCommand < Gem::Command
   include Gem::VersionOption
   include Gem::LocalRemoteOptions
   include Gem::InstallUpdateOptions
+  include Gem::UpdateSuggestion
 
   def initialize
     defaults = Gem::DependencyInstaller::DEFAULT_OPTIONS.merge({
@@ -45,7 +48,7 @@ class Gem::Commands::InstallCommand < Gem::Command
   end
 
   def defaults_str # :nodoc:
-    "--both --version '#{Gem::Requirement.default}' --no-force\n" +
+    "--both --version '#{Gem::Requirement.default}' --no-force\n" \
       "--install-dir #{Gem.dir} --lock\n" +
       install_update_defaults_str
   end
@@ -168,6 +171,8 @@ You can use `i` command instead of `install`.
 
     show_installed
 
+    say update_suggestion if eglible_for_update?
+
     terminate_interaction exit_code
   end
 
@@ -258,7 +263,7 @@ You can use `i` command instead of `install`.
     return unless errors
 
     errors.each do |x|
-      return unless Gem::SourceFetchProblem === x
+      next unless Gem::SourceFetchProblem === x
 
       require_relative "../uri"
       msg = "Unable to pull data from '#{Gem::Uri.redact(x.source.uri)}': #{x.error.message}"
